*   Simple program of output to a file.
*---+----1----+----2----+----3----+----4----+----5----+----6----+----7--
      implicit none

*  -- constants
      character *(*) fname
      parameter (fname = 'ex4.dat')
      integer    ndata
      parameter (ndata = 20)

*  -- local vars
      integer mode
      real*8  x(0:ndata), y(0:ndata)
*-----------------------------------------------------------------------
      write(*,*) 'Read from or write to "', fname, '"'
      write(*,*) 'read:0 or write:1 ?'
      read(*,*)  mode
      
      if(mode.eq.0) then
        call datrd(fname, ndata, x, y)
      else if(mode.eq.1) then
        call gendat(ndata, x, y)
        call datwrt(fname, ndata, x, y)
      else
        write(*,*) '*** illegal mode = ', mode
        stop 999
      endif

      call plot(ndata, x, y)
      end
*=======================================================================
      subroutine datwrt(fname, ndata, x, y)
      implicit none

*  -- args
      character *(*) fname
      integer  ndata
      real*8   x(0:ndata), y(0:ndata)

*  -- constants and variables
      integer    funit
      parameter (funit = 1)
      integer    ios
*-----------------------------------------------------------------------
      write(*,*) 'write date to ', fname
      open(funit, file=fname, status='new', 
     &     form='unformatted', iostat=ios)
      if(ios.ne.0) then
        write(*,*) '*** error for writing data to new file: ios=',ios
        write(*,*) '*** ', fname, ' may be exisiting.'
        stop 999
      endif
      rewind(funit)
      write(funit) x, y
      close(funit)
      return
      end
*=======================================================================
      subroutine datrd(fname, ndata, x, y)
      implicit none

*  -- args
      character *(*) fname
      integer  ndata
      real*8   x(0:ndata), y(0:ndata)

*  -- constants and variables
      integer    funit
      parameter (funit = 1)
      integer    ios
*-----------------------------------------------------------------------
      write(*,*) 'read date from ', fname
      open(funit, file=fname, status='old', 
     &     form='unformatted', iostat=ios)
      if(ios.ne.0) then
        write(*,*) '*** error for reading data from file: ios=',ios
        write(*,*) '*** ', fname, ' may not be exisiting.'
        stop 999
      endif
      rewind(funit)
      read(funit) x, y
      return
      end
*=======================================================================
      subroutine gendat(ndata, x, y)
      implicit none

*  -- args
      integer  ndata
      real*8   x(0:ndata), y(0:ndata)

*  -- parameters for fncl
      real*8   fncl
      integer  npx, npl
      common  /idat/npx, npl

*  -- vars
      real*8   d
      integer  k
*-----------------------------------------------------------------------
      npx = 1
      npl = 1
      d = 1/dble(ndata)
      do k = 0, ndata
        x(k) = k*d
        y(k) = fncl(x(k))
      enddo
      return
      end
*=======================================================================
      subroutine plot(ndata, x, y)
      implicit none

*  -- args
      integer  ndata
      real*8   x(0:ndata), y(0:ndata)

*  -- vars
      integer  j, k, ky
      real*8   x0, x1, y0, y1, a0, a1
*-----------------------------------------------------------------------
      y0 = y(0)
      y1 = y(0)
      do k = 1, ndata
        if(y0.gt.y(k)) then
          y0 = y(k)
          x0 = x(k)
        endif
        if(y1.lt.y(k)) then
          y1 = y(k)
          x1 = x(k)
        endif
      enddo
      if(y0.eq.y1) then
        write(*,*) 'constant y = ', y0
        return
      endif
      write(*,*) 'min: f(',x0,') = ', y0, '  max: f(',x1,') = ', y1
      a1 = 50/(y1-y0)
      a0 = - y0*a1
      do k = 0, ndata
        ky = a0 + a1*y(k)
        ky = min(max(0, ky), 50)
        write(*,'(f8.4,'':'',52a)') x(k), (' ', j=0,ky-1), '*'
      enddo
      return
      end
*=======================================================================
      double precision function fncl(x)
*
*     fncl(x) = x**npx * log(x)**npl       for npl >= 0, x >= 0
*
*     Input
*       x : real number for which the function is calculated
*       npx, npl : selector of a function.  This parameter is passed
*                  through common block /idat/
*     Output
*       calculated function value

      implicit none

*  -- argument of the function
      double precision x

*  -- parameters
      integer    npx, npl
      common    /idat/npx, npl
*-----------------------------------------------------------------------

*  -- check 
      if(x.lt.0) then
        write(*,*) 'log(x), x = ',x,' < 0'
        stop 999
      endif

*    -- special point x=0 ( x*log(x)=0 for x=0 )
      if(npx.gt.0 .and. x.eq.0) then
        fncl = 0
      else
        fncl = x**npx * log(x)**npl
      endif

      return
      end
