/* plot t = - log(1-z),  z = r exp(i \theta),  0 < r < 1
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int main();
double func(double x);
FILE *cfopen(FILE *fpout, char *fname);
FILE *cfclose(FILE *fpout);
void makefile(char *fname0, char *fname1);

#define float(x) ((double) x)

#define XLIM (1.2)
#define YLIM (1.2)

int    pmode = 0;
double xlim0 = 1 - XLIM, xlim1 = XLIM;
double ylim0 = - YLIM, ylim1 = YLIM - 1;

int main()
{
    char *fname0 = "sample.dat";
    char *fname1 = "sample1.dat";
    int   nn   = 100;
    FILE *fpout = NULL;
    int   j;
    double x;
    double xylim;

    fpout = cfopen(fpout, fname0);
    for(j = 0; j <= nn; j += 1) {
        x  = j/float(nn);
        fprintf(fpout, "%g %g\n", x, func(x));
    }
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, fname1);
    x = exp(-1.0);
    fprintf(fpout, "%g %g\n", x, 0.0);
    fprintf(fpout, "%g %g\n", x, func(x));
    fpout = cfclose(fpout);
 

    makefile(fname0, fname1);
    return 0;
}

double func(double x) {
    if(x == 0) {
        return 0;
    } else {
        return x * log(fabs(x));
    }
}

FILE *cfclose(FILE *fpout)
{
    fclose(fpout);
    return NULL;
}

FILE *cfopen(FILE *fpout, char *fname)
{
    if(fpout != NULL) {
        fpout = cfclose(fpout);
    }
    if((fpout = fopen(fname, "w")) == NULL) {
        fprintf(stderr, "*** %s : cannot open\n", fname);
        exit(1);
    }
    return fpout;
}

void makefile(char *fname0, char *fname1)
{
    FILE *fpout = NULL;

    fpout = cfopen(fpout, "Makefile.run");
    fprintf(fpout,"LATEX\t= platex\n");
    fprintf(fpout,"GNUPLOT\t= gnuplot\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"entry: x\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"all: x tex ps\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"tex: main.dvi\n");
    fprintf(fpout,"\txdvi main\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"ps: plot.ps\n");
    fprintf(fpout,"\tghostview plot.ps\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"main.dvi: main.tex plot.tex\n");
    fprintf(fpout,"\t${LATEX} main\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"x: plot.gnp %s %s\n", fname0, fname1);
    fprintf(fpout,"\t${GNUPLOT} plot.gnp\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"plot.tex: tex.gnp %s %s\n", fname0, fname1);
    fprintf(fpout,"\t${GNUPLOT} tex.gnp\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"plot.ps: ps.gnp %s %s\n", fname0, fname1);
    fprintf(fpout,"\t${GNUPLOT} ps.gnp\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"plot.png: png.gnp %s %s\n", fname0, fname1);
    fprintf(fpout,"\t${GNUPLOT} png.gnp\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"clean:\n");
    fprintf(fpout,"\trm -f plot.tex *.dvi *.aux *.log *.ps t tt\n");
    fprintf(fpout,"\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "main.gnp");
    fprintf(fpout,"set title \"mode=%d\"\n", pmode);
    if(pmode >= 0) {
        fprintf(fpout,"set size square\n");
        fprintf(fpout,"set xrange [%g:%g]\n", xlim0, xlim1);
        fprintf(fpout,"set yrange [%g:%g]\n", ylim0, ylim1);
    }
    fprintf(fpout,"set nokey\n");
    fprintf(fpout,"set parametric\n");
    fprintf(fpout,"set multiplot\n");
    fprintf(fpout,"plot '%s' title 'data0' with lines\n", fname0);
    fprintf(fpout,"plot '%s' title 'data1' with lines\n", fname1);
    fprintf(fpout,"set nomultiplot\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "main.tex");
    fprintf(fpout,"\\documentclass[12pt]{article}\n");
    fprintf(fpout,"\\pagestyle{empty}\n");
    fprintf(fpout,"\\begin{document}\n");
    fprintf(fpout,"%%----------------------------------------------------\n");
    fprintf(fpout,"%%\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"\\begin{figure}[htb] \n");
    fprintf(fpout,"\\unitlength 1mm\n");
    fprintf(fpout,"\\include{plot}\n");
    fprintf(fpout,"\\caption{figure caption}\n");
    fprintf(fpout,"\\end{figure}\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"%%\n");
    fprintf(fpout,"\n");
    fprintf(fpout,"\\end{document}\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "plot.gnp");
    fprintf(fpout,"load 'main.gnp'\n");
    fprintf(fpout,"pause -1 \"Hit return to end\"\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "png.gnp");
    fprintf(fpout,"set term png\n");
    fprintf(fpout,"set output 'plot.png'\n");
    fprintf(fpout,"load 'main.gnp'\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "ps.gnp");
    fprintf(fpout,"set term postscript\n");
    fprintf(fpout,"set output 'plot.ps'\n");
    fprintf(fpout,"load 'main.gnp'\n");
    fpout = cfclose(fpout);

    fpout = cfopen(fpout, "tex.gnp");
    fprintf(fpout,"set term pslatex\n");
    fprintf(fpout,"set output 'plot.tex'\n");
    fprintf(fpout,"load 'main.gnp'\n");
    fpout = cfclose(fpout);
}
